package client.drivers.IPCamera;

import java.awt.Image;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageDecoder;

/**
 * Driver for IPCamera
 * Consists of two parts:
 * 1: A thread is created that continuously decode the JPEG images in the MJPEG stream.
 * Whenever a JPEG image is successfully decoded, an event containing the image is fired.
 * 2: Public methods for moving the camera.
 */
public class IPCameraDriver implements Runnable {
	private DataInputStream cameraStream = null;
	private String controlURL;
	int lastCommand;
	private Thread thread;
	private IPCameraInterface jpegDecodedListener;
		
	/**
	 * The constructor opens the data stream for the camera, and starts the thread that decodes JPEGs from the stream.
	 * @param address IP-address of the IPCamera.
	 * @param user username of the IPCamera.
	 * @param pass password of the IPCamera.
	 */
	public IPCameraDriver(String address, String user, String pass){
		URL url;
		BufferedInputStream buffer=null;
		
		//Connect to the video stream and create DataInputStream object		
		try {
			url = new URL(address + "videostream.cgi?user="+user+"&pwd="+pass);
			buffer = new BufferedInputStream(url.openStream());
		} catch (MalformedURLException e){
			e.printStackTrace();
		} catch (IOException e){
			e.printStackTrace();
		}
		cameraStream = new DataInputStream(buffer);

		//Create URL for use to control camera
		controlURL = address + "decoder_control.cgi?user="+user+"&pwd="+pass+"&command=";
		
		//Create and start thread for streaming
		thread = new Thread(this);
		thread.start();
	}
	
	/**
	 * Disconnect the stream.
	 */
	public void disconnect(){
		//Currently not used
		try	{
			cameraStream.close();
		} catch(Exception e){;}
	}

	/**
	 * Register event listener with the driver, only one listener is supported.
	 * @param listener the object that listens for JPEGs decoded by the driver.
	 */
	public void addJPEGDecodedListener(IPCameraInterface listener) {
		jpegDecodedListener = listener;
	}
	
	/**
	 * Remove the currently registered listener. 
	 */
	public void removeJPEGDecodedListener() {
		jpegDecodedListener=null;
	}
	
	/**
	 * Fire an event containing a JPEG image.
	 * @param jpeg the image that are sent with the event.
	 */
	private void fireJPEGDecoded(Image jpeg){
		JPEGDecodedEvent evt = new JPEGDecodedEvent(this, jpeg);
		
		//Send event to listener if registered
		if(jpegDecodedListener!=null){
			jpegDecodedListener.jpegDecoded(evt);
		}
	}
	
	/**
	 * This method is running as a thread. It tries to decode JPEG images from the data stream, 
	 * and fire an event when one is successfully decoded. 
	 */
	@SuppressWarnings("deprecation")
	public void run(){
		Image jpeg;
		
		//Runs as thread
		
		while(true){ //Infinite loop
			try {
				//Wait for line that indicate start of MJPEG packet
				while(!cameraStream.readLine().equals("--ipcamera"));
				//Drop the three next lines, the 4th will be the first line of the JPEG
				cameraStream.readLine();
				cameraStream.readLine();
				cameraStream.readLine();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			//Decode and save the JPEG image
			jpeg = readJPG();
			
			//Fire event if a JPEG was successfully read from the stream
			if(jpeg != null) fireJPEGDecoded(jpeg);
		}
	}
	
	/**
	 * Decode a JPEG image from the stream. The stream must be at the point where the JPEG starts
	 * @return Image the decoded JPEG image from the stream. Will be null if the decoding was unsuccessful.
	 */
	private Image readJPG(){
		Image image = null;
		try {
			//Decode JPEG image from the stream
			JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(cameraStream);
			image = decoder.decodeAsBufferedImage();
		}catch(Exception e){
			System.out.println("feil p JPEG");
			e.printStackTrace();
		}
		
		//Return image, will be null if decoding was unsuccessful
		return image;
	}
	
	/**
	 * Method for moving the camera.
	 * @param command the type of movement. Start up=0, stop up=1, start down=2, stop down=3, start left=4, 
	 * stop left=5, start right=6, stop right=7.
	 * @return true if command was sent successfully.
	 */
	private boolean performControl(int command){
		URL url = null;
		BufferedReader controlStream = null;
		String inputLine;
		
		try {
			//Creating URL with the correct command
			url = new URL(controlURL+""+command);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		
		try {
			//Connect to the URL
			controlStream = new BufferedReader(new InputStreamReader(url.openStream()));
		} catch (MalformedURLException e) {
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			System.out.println("unable to connect");
			e.printStackTrace();
			return false;
		}
		
		try {
			//Print result, should be "ok."
			while((inputLine = controlStream.readLine()) != null) System.out.println(inputLine);
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		try {
			//Close connection
			controlStream.close();
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	/**
	 * Public method for giving starting moving up
	 */
	public void goUp(){
		if(performControl(0)) lastCommand = 0;
	}
	
	/**
	 * Public method for giving stopping moving up
	 */
	public void stopUp(){
		if(performControl(1)) lastCommand = 1;
	}
	
	/**
	 * Public method for giving starting moving down
	 */
	public void goDown(){
		if(performControl(2)) lastCommand = 2;
	}
	
	/**
	 * Public method for giving stopping moving down
	 */
	public void stopDown(){
		if(performControl(3)) lastCommand = 3;
	}
	
	/**
	 * Public method for giving starting moving left
	 */
	public void goLeft(){
		if(performControl(4)) lastCommand = 4;
	}
	
	/**
	 * Public method for giving stopping moving left
	 */
	public void stopLeft(){
		if(performControl(5)) lastCommand = 5;
	}
	
	/**
	 * Public method for giving starting moving right
	 */
	public void goRight(){
		if(performControl(6)) lastCommand = 6;
	}
	
	/**
	 * Public method for giving stopping moving right
	 */
	public void stopRight(){
		if(performControl(7)) lastCommand = 7;
	}
	
	/**
	 * Public method for stopping all movement
	 */
	public void stopAll(){
		stopUp();
		stopDown();
		stopLeft();
		stopRight();
//		if(lastCommand != -1){
//			if(performCommand(lastCommand+1)) lastCommand = -1;
//		}
	}
	
	
}
